#ifndef AnimState_H_
#define AnimState_H_

#include <irrlicht.h>
//#include <irrKlang.h>
//#include <IGUIEnvironment.h>
//#include <IGUIButton.h>
#include "Newton.h"


using namespace irr;
using namespace core;
using namespace scene;
using namespace video;


//using namespace irrklang;
struct SObject
{
//Irrlicht scene node
IMeshSceneNode *irr_node;

//Newton collision object
NewtonCollision *nwtn_collision;

};

class AnimState : public State<Game, irr::SEvent>
{

private:
    //objekti
    SObject sphere;//kirni.3ds
    SObject cube;  //pile.3ds
    //animacija.x
    IAnimatedMesh* mesh;
    IAnimatedMeshSceneNode* node;
    //anikacija.x

    //newton svet
    NewtonWorld *nWorld;
    //kamera
	ICameraSceneNode *cam;

    float vel;

public:

	AnimState();

	virtual ~AnimState();

	virtual void onEnter(Game* game);

	virtual void onLeave(Game* game);

	virtual void onUpdate(Game* game);

	virtual const bool onEvent(Game* entity, const irr::SEvent& event);

	NewtonCollision* CreateCollisionFromMesh(NewtonWorld*, IMesh*);

	NewtonCollision* CreateTreeCollisionFromMesh(NewtonWorld*, IMesh* );

	bool CheckForCollision(NewtonWorld*, SObject*, SObject*);


};

AnimState::AnimState()
{
}

AnimState::~AnimState()
{
}

void AnimState::onEnter(Game* game)
{
    vel=0.5f;

    cam = game->getSceneManager()->addCameraSceneNodeFPS();
	cam->setPosition(vector3df(0, 0, 200));

    nWorld = NewtonCreate(NULL,NULL);

// set 0 for exact collision detection,
// 1 or n for use in applications where speed is more important
    NewtonSetSolverModel(nWorld,1);

        //Load mesh and create new irrlicht node
sphere.irr_node = game->getSceneManager()->addMeshSceneNode(game->getSceneManager()->getMesh("../../media/kirbi.3ds")->getMesh(0));
cube.irr_node = game->getSceneManager()->addMeshSceneNode(game->getSceneManager()->getMesh("../../media/pile.3ds")->getMesh(0));

//ucitaj animaciju iz .x
sphere.irr_node->getMaterial(0).EmissiveColor.set(255,0,255,5);
cube.irr_node->getMaterial(0).EmissiveColor.set(255,0,255,255);

mesh = game->getSceneManager()->getMesh("../../media/animacija.x");
node = game->getSceneManager()->addAnimatedMeshSceneNode( mesh );

 if (node)
        {
                node->setPosition(vector3df(-200,0,0));
                node->setMaterialFlag(EMF_LIGHTING, false);
                node->setScale(vector3df(30,30,30));
                node->setAnimationSpeed(50);
        }

//Now create collision mesh from irrlicht mesh
sphere.nwtn_collision = CreateCollisionFromMesh(nWorld,sphere.irr_node->getMesh());
cube.nwtn_collision = CreateCollisionFromMesh(nWorld,cube.irr_node->getMesh());

//Set position
sphere.irr_node->setPosition(vector3df(-100,0,0));
cube.irr_node->setPosition(vector3df(0,0,0));

}

void AnimState::onLeave(Game* game)
{

}

const bool AnimState::onEvent(Game* game, const irr::SEvent& event)
{
    if (event.EventType == irr::EET_KEY_INPUT_EVENT)
	{
		if (!event.KeyInput.PressedDown)
		{
			switch (event.KeyInput.Key)
			{
				case irr::KEY_ESCAPE: game->setState(game->findGameState("intro")); return true;

				default: ;
			}
		}
	}
    return false;
}

void AnimState::onUpdate(Game* game)
{
    sphere.irr_node->setPosition(sphere.irr_node->getPosition()+vector3df(1,0,0)*vel);

        if(sphere.irr_node->getPosition().X > 5.5f) vel = -0.05f;
        if(sphere.irr_node->getPosition().X < -5.5f) vel = 0.05f;

    //check for collision and set red color if collision detected
    if(CheckForCollision(nWorld,&sphere,&cube))
    {
            cube.irr_node->getMaterial(0).EmissiveColor.set(255,255,0,0);
            game->getSoundEngine()->play2D("../../media/bell.wav");
    }
    else //use blue color
    {
        cube.irr_node->getMaterial(0).EmissiveColor.set(255,0,0,255);
    }
}


NewtonCollision *AnimState::CreateCollisionFromMesh(NewtonWorld *nWorld, IMesh *irr_mesh )
{
int nMeshBuffer = 0; //Mesh Buffer count
IMeshBuffer *mesh_buffer = NULL;
float *vertices; //Array to store vertices

u32 nVertices = 0;

//Get number of vertices
for( nMeshBuffer=0 ; nMeshBuffer < irr_mesh->getMeshBufferCount() ; nMeshBuffer++ )
{
nVertices += irr_mesh->getMeshBuffer(nMeshBuffer)->getVertexCount();
}


//create buffer for vertices
vertices = new float[nVertices * 3];
u32 tmpCounter = 0;

//Get (irr_)mesh buffers and copy face vertices
for( nMeshBuffer=0 ; nMeshBuffer < irr_mesh->getMeshBufferCount() ; nMeshBuffer++ )
{
mesh_buffer = irr_mesh->getMeshBuffer(nMeshBuffer);

//Get pointer to vertices and indices
S3DVertex *S3vertices = (S3DVertex*)mesh_buffer->getVertices();


//copy vertices from mesh to buffer
for(int i=0; i<mesh_buffer->getVertexCount(); i++)
{
vertices[tmpCounter++] = S3vertices[i].Pos.X;
vertices[tmpCounter++] = S3vertices[i].Pos.Y;
vertices[tmpCounter++] = S3vertices[i].Pos.Z;
}
}

//Create Newton collision object
NewtonCollision *collision_obj = NewtonCreateConvexHull(nWorld,nVertices,vertices,sizeof(float)*3,NULL);

//delete vertices
delete [] vertices;

return collision_obj;

}

NewtonCollision *AnimState::CreateTreeCollisionFromMesh(NewtonWorld *nWorld, IMesh *irr_mesh )
{
//Create new (tree optimized) collision mesh
NewtonCollision *collision_obj = NewtonCreateTreeCollision(nWorld ,NULL);

//Begin collision mesh construction
NewtonTreeCollisionBeginBuild(collision_obj);

int nMeshBuffer = 0; //Mesh Buffer count
int v_index[3] = {0,0,0}; //vertex indices
IMeshBuffer *mesh_buffer = NULL;
float array[9]; //Array to store 3 vertices

//Get (irr_)mesh buffers and copy face by face to collision mesh
for( nMeshBuffer=0 ; nMeshBuffer < irr_mesh->getMeshBufferCount() ; nMeshBuffer++ )
{
mesh_buffer = irr_mesh->getMeshBuffer(nMeshBuffer);

//Get pointer to vertices and indices
S3DVertex *vertices = (S3DVertex*)mesh_buffer->getVertices();
u16 *indices = mesh_buffer->getIndices();

//Fill collision mesh
for(int i=0; i<mesh_buffer->getIndexCount(); i+=3)
{
v_index[0] = indices[ i ];
v_index[1] = indices[i+1];
v_index[2] = indices[i+2];

// 1st position vertex
array[0] = vertices[ v_index[0] ].Pos.X;
array[1] = vertices[ v_index[0] ].Pos.Y;
array[2] = vertices[ v_index[0] ].Pos.Z;

// 2nd position vertex
array[3] = vertices[ v_index[1] ].Pos.X;
array[4] = vertices[ v_index[1] ].Pos.Y;
array[5] = vertices[ v_index[1] ].Pos.Z;

// 3rd position vertex
array[6] = vertices[ v_index[2] ].Pos.X;
array[7] = vertices[ v_index[2] ].Pos.Y;
array[8] = vertices[ v_index[2] ].Pos.Z;

//Add new face to collision mesh
NewtonTreeCollisionAddFace(collision_obj, //collision mesh to add face to
3, //number of vertices in array
(float*)array, //pointer to vertex array
3*sizeof(float),//size of each vertex
1); //ID of the face
}
}
//End collision contruction , set 1 as 2dn param for optimization
NewtonTreeCollisionEndBuild(collision_obj,0);

return collision_obj;

}

bool AnimState::CheckForCollision(NewtonWorld *nWorld, SObject *obj_A, SObject *obj_B)
{
//Matrix to store irr_node position
matrix4 mat_A,mat_B;

//Copy position
mat_A.makeIdentity();
mat_B.makeIdentity();

mat_A.setTranslation(obj_A->irr_node->getPosition());
mat_B.setTranslation(obj_B->irr_node->getPosition());

const int nContacts = 2;
float contacts[3 * nContacts];
float normals[3 * nContacts];
float penetration[ nContacts ];

//Check for collision between collision meshes,
// returns number of contact points
int nHits = NewtonCollisionCollide( nWorld,nContacts,
obj_A->nwtn_collision, (float*)&mat_A[0],
obj_B->nwtn_collision, (float*)&mat_B[0],
contacts,
normals,
penetration);
//Collision detected if nHits > 0
if( nHits > 0)
return true;
return false;

}

#endif /*AnimState_H_*/

